/**
 * name: @tresjs/post-processing
 * version: v1.0.0-next.1
 * (c) 2024
 * description: Post-processing library for TresJS
 * author: Alvaro Saburido <hola@alvarosaburido.dev> (https://github.com/alvarosabu/)
 */
import { inject as q, shallowRef as _, watchEffect as B, onUnmounted as L, watch as h, defineComponent as d, provide as H, computed as $, renderSlot as I } from "vue";
import { EffectPass as K, BloomEffect as P, DepthOfFieldEffect as R, EffectComposer as O, RenderPass as J, NormalPass as Q, DepthDownsamplingPass as Z, GlitchEffect as w, GlitchMode as ee, OutlineEffect as D, PixelationEffect as T, VignetteTechnique as te, BlendFunction as G, VignetteEffect as z, NoiseEffect as k } from "postprocessing";
import { useTresContext as b, useLoop as ne, normalizeColor as E } from "@tresjs/core";
import { HalfFloatType as A } from "three";
const W = Symbol("effectComposer"), p = (l) => {
  const n = q(W), e = _(null), t = _(null), { scene: o, camera: a } = b();
  B(() => {
    !a.value || !(t != null && t.value) || (t.value.mainCamera = a.value);
  });
  let i = () => {
  };
  return i = B(() => {
    !a.value || !(n != null && n.value) || !o.value || (i(), !t.value && (t.value = l(), e.value = new K(a.value, t.value), n.value.addPass(e.value)));
  }), L(() => {
    var s, u, f;
    e.value && ((s = n == null ? void 0 : n.value) == null || s.removePass(e.value)), (u = t.value) == null || u.dispose(), (f = e.value) == null || f.dispose();
  }), {
    pass: e,
    effect: t
  };
}, j = /([^[.\]])+/g, oe = (l, n) => {
  if (!n)
    return;
  const e = Array.isArray(n) ? n : n.match(j);
  return e == null ? void 0 : e.reduce((t, o) => t && t[o], l);
}, M = (l, n, e) => {
  const t = Array.isArray(n) ? n : n.match(j);
  t && t.reduce((o, a, i) => (o[a] === void 0 && (o[a] = {}), i === t.length - 1 && (o[a] = e), o[a]), l);
}, x = (l, n) => {
  const e = { ...l };
  return n.forEach((t) => delete e[t]), e;
}, X = (l, n, e, t, o = {}) => h(l, (a) => {
  var i;
  if (n.value)
    if (a === void 0) {
      const s = t();
      M(n.value, e, oe(s, e)), (i = s.dispose) == null || i.call(s);
    } else
      M(n.value, e, l());
}, o), y = (l, n, e) => l.map(([t, o]) => X(
  t,
  n,
  o,
  e
)), C = (l, n, e) => Object.keys(l).map((t) => X(
  () => l[t],
  n,
  t,
  e
)), ie = /* @__PURE__ */ d({
  __name: "Bloom",
  props: {
    blendFunction: {},
    intensity: {},
    kernelSize: {},
    luminanceThreshold: {},
    luminanceSmoothing: {},
    mipmapBlur: { type: Boolean, default: void 0 }
  },
  setup(l, { expose: n }) {
    const e = l, { pass: t, effect: o } = p(() => new P(e));
    return n({ pass: t, effect: o }), y(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.intensity, "intensity"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.luminanceSmoothing, "luminanceMaterial.smoothing"],
        [() => e.luminanceThreshold, "luminanceMaterial.threshold"]
      ],
      o,
      () => new P()
    ), () => {
    };
  }
}), ue = /* @__PURE__ */ d({
  __name: "DepthOfField",
  props: {
    blendFunction: {},
    worldFocusDistance: {},
    worldFocusRange: {},
    focusDistance: {},
    focusRange: {},
    bokehScale: {},
    resolutionScale: {},
    resolutionX: {},
    resolutionY: {}
  },
  setup(l, { expose: n }) {
    const e = l, { camera: t } = b(), { pass: o, effect: a } = p(() => new R(t.value, e));
    return n({ pass: o, effect: a }), y(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.worldFocusDistance, "circleOfConfusionMaterial.worldFocusDistance"],
        [() => e.focusDistance, "circleOfConfusionMaterial.focusDistance"],
        [() => e.worldFocusRange, "circleOfConfusionMaterial.worldFocusRange"],
        [() => e.focusRange, "circleOfConfusionMaterial.focusRange"],
        [() => e.bokehScale, "bokehScale"],
        [() => e.resolutionScale, "blurPass.resolution.scale"],
        [() => e.resolutionX, "resolution.width"],
        [() => e.resolutionY, "resolution.height"]
      ],
      a,
      () => new R()
    ), () => {
    };
  }
});
let v;
function N() {
  var l;
  if (v !== void 0)
    return v;
  try {
    let n;
    const e = document.createElement("canvas");
    return v = !!(window.WebGL2RenderingContext && (n = e.getContext("webgl2"))), n && ((l = n.getExtension("WEBGL_lose_context")) == null || l.loseContext()), v;
  } catch {
    return v = !1;
  }
}
const ce = /* @__PURE__ */ d({
  __name: "EffectComposer",
  props: {
    enabled: { type: Boolean, default: !0 },
    children: {},
    depthBuffer: { type: Boolean, default: void 0 },
    disableNormalPass: { type: Boolean, default: !1 },
    stencilBuffer: { type: Boolean, default: void 0 },
    resolutionScale: {},
    autoClear: { type: Boolean, default: !0 },
    multisampling: { default: 0 },
    frameBufferType: { default: A }
  },
  emits: ["render"],
  setup(l, { expose: n, emit: e }) {
    const t = l, o = e, { scene: a, camera: i, renderer: s, sizes: u, render: f } = b(), r = _(null);
    let S = null, g = null;
    H(W, r), n({ composer: r });
    const Y = () => {
      r.value && (g = new Q(a.value, i.value), g.enabled = !1, r.value.addPass(g), t.resolutionScale !== void 0 && N() && (S = new Z({
        normalBuffer: g.texture,
        resolutionScale: t.resolutionScale
      }), S.enabled = !1, r.value.addPass(S)));
    }, V = $(() => {
      const c = new O(), m = {
        depthBuffer: t.depthBuffer !== void 0 ? t.depthBuffer : c.inputBuffer.depthBuffer,
        stencilBuffer: t.stencilBuffer !== void 0 ? t.stencilBuffer : c.inputBuffer.stencilBuffer,
        multisampling: N() ? t.multisampling !== void 0 ? t.multisampling : c.multisampling : 0,
        frameBufferType: t.frameBufferType !== void 0 ? t.frameBufferType : A
      };
      return c.dispose(), m;
    }), F = () => {
      !s.value && !a.value && !i.value || (r.value = new O(s.value, V.value), r.value.addPass(new J(a.value, i.value)), t.disableNormalPass || Y());
    };
    h([s, a, i, () => t.disableNormalPass], () => {
      !u.width.value || !u.height.value || F();
    }), h(() => [u.width.value, u.height.value], ([c, m]) => {
      !c && !m || (r.value ? r.value.setSize(c, m) : F());
    }, {
      immediate: !0
    });
    const { render: U } = ne();
    return U(() => {
      if (t.enabled && s.value && r.value && u.width.value && u.height.value && f.frames.value > 0) {
        const c = s.value.autoClear;
        s.value.autoClear = t.autoClear, t.stencilBuffer && !t.autoClear && s.value.clearStencil(), r.value.render(), o("render", r.value), s.value.autoClear = c;
      }
      f.priority.value = 0, f.mode.value === "always" ? f.frames.value = 1 : f.frames.value = Math.max(0, f.frames.value - 1);
    }), L(() => {
      var c;
      (c = r.value) == null || c.dispose();
    }), (c, m) => I(c.$slots, "default");
  }
}), fe = /* @__PURE__ */ d({
  __name: "Glitch",
  props: {
    blendFunction: {},
    delay: {},
    duration: {},
    strength: {},
    mode: {},
    active: { type: Boolean },
    ratio: {},
    columns: {},
    chromaticAberrationOffset: {},
    perturbationMap: {},
    dtSize: {}
  },
  setup(l, { expose: n }) {
    const e = l, { pass: t, effect: o } = p(() => new w(e));
    return n({ pass: t, effect: o }), B(() => {
      const a = () => {
        if (e.mode !== void 0)
          return e.active === !1 ? ee.DISABLED : e.mode;
        const i = new w(), s = i.mode;
        return i.dispose(), s;
      };
      o.value && (o.value.mode = a());
    }), C(
      x(e, ["active", "mode", "blendFunction"]),
      o,
      () => new w()
    ), () => {
    };
  }
}), de = /* @__PURE__ */ d({
  __name: "Outline",
  props: {
    outlinedObjects: {},
    blur: { type: Boolean, default: void 0 },
    xRay: { type: Boolean, default: void 0 },
    kernelSize: {},
    pulseSpeed: {},
    resolutionX: {},
    resolutionY: {},
    edgeStrength: {},
    patternScale: {},
    multisampling: {},
    blendFunction: {},
    patternTexture: {},
    resolutionScale: {},
    hiddenEdgeColor: {},
    visibleEdgeColor: {}
  },
  setup(l, { expose: n }) {
    const e = l, t = (r) => r !== void 0 ? E(r).getHex() : void 0, { camera: o, scene: a } = b(), i = {
      blur: e.blur,
      xRay: e.xRay,
      kernelSize: e.kernelSize,
      pulseSpeed: e.pulseSpeed,
      resolutionX: e.resolutionX,
      resolutionY: e.resolutionY,
      patternScale: e.patternScale,
      edgeStrength: e.edgeStrength,
      blendFunction: e.blendFunction,
      multisampling: e.multisampling,
      patternTexture: e.patternTexture,
      resolutionScale: e.resolutionScale,
      hiddenEdgeColor: t(e.hiddenEdgeColor),
      visibleEdgeColor: t(e.visibleEdgeColor)
    }, { pass: s, effect: u } = p(() => new D(a.value, o.value, i));
    n({ pass: s, effect: u }), h(
      [() => e.outlinedObjects, u],
      // watchEffect is intentionally not used here as it would result in an endless loop
      () => {
        var r;
        (r = u.value) == null || r.selection.set(e.outlinedObjects || []);
      },
      {
        immediate: !0
      }
    );
    const f = $(() => ({
      hiddenEdgeColor: e.hiddenEdgeColor ? E(e.hiddenEdgeColor) : void 0,
      visibleEdgeColor: e.visibleEdgeColor ? E(e.visibleEdgeColor) : void 0
    }));
    return y(
      [
        /* some properties are not updated because of different reasons:
            resolutionX - has no setter in OutlineEffect
            resolutionY - has no setter in OutlineEffect
            blendFunction - has no setter in OutlineEffect
            patternTexture - different type in constructor and in setter
            resolutionScale - has no setter in OutlineEffect
          */
        [() => e.blur, "blur"],
        [() => e.xRay, "xRay"],
        [() => e.pulseSpeed, "pulseSpeed"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.edgeStrength, "edgeStrength"],
        [() => e.patternScale, "patternScale"],
        [() => e.multisampling, "multisampling"],
        [() => f.value.hiddenEdgeColor, "hiddenEdgeColor"],
        [() => f.value.visibleEdgeColor, "visibleEdgeColor"]
      ],
      u,
      () => new D()
    ), () => {
    };
  }
}), pe = /* @__PURE__ */ d({
  __name: "Pixelation",
  props: {
    granularity: {}
  },
  setup(l, { expose: n }) {
    const e = l, { pass: t, effect: o } = p(() => new T(e.granularity));
    return n({ pass: t, effect: o }), C(
      e,
      o,
      () => new T()
    ), () => {
    };
  }
}), me = /* @__PURE__ */ d({
  __name: "Vignette",
  props: {
    technique: { default: te.DEFAULT },
    blendFunction: { default: G.NORMAL },
    offset: { default: 0.5 },
    darkness: { default: 0.5 }
  },
  setup(l, { expose: n }) {
    const e = l, { pass: t, effect: o } = p(() => new z(e));
    return n({ pass: t, effect: o }), C(
      x(e, ["blendFunction"]),
      o,
      () => new z()
    ), () => {
    };
  }
}), ve = /* @__PURE__ */ d({
  __name: "Noise",
  props: {
    premultiply: { type: Boolean, default: !1 },
    blendFunction: { default: G.SCREEN }
  },
  setup(l, { expose: n }) {
    const e = l, { pass: t, effect: o } = p(() => new k(e));
    return n({ pass: t, effect: o }), C(
      x(e, ["blendFunction"]),
      o,
      () => new k()
    ), () => {
    };
  }
});
export {
  ie as Bloom,
  ue as DepthOfField,
  ce as EffectComposer,
  fe as Glitch,
  ve as Noise,
  de as Outline,
  pe as Pixelation,
  me as Vignette,
  p as useEffect
};
